@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 
 #include "ipecanvas_qt.h"
 
+#include <QGuiApplication>
 #include <QMenu>
 #include <QContextMenuEvent>
 #include <QPainter>
@@ -70,21 +71,34 @@ void LayerBox::set(const Page *page, int view)
   page->objectsPerLayer(objCounts);
   iInSet = true;
   clear();
+  QPalette palette = QGuiApplication::palette();
   for (int i = 0; i < page->countLayers(); ++i) {
     QString text = QString("%1 (%2)").arg(QIpe(page->layer(i))).arg(objCounts[i]);
     QListWidgetItem *item = new LayerItem(text, page->layer(i), this);
     item->setFlags(Qt::ItemIsUserCheckable|Qt::ItemIsEnabled);
     item->setCheckState(page->visible(view, i) ? Qt::Checked : Qt::Unchecked);
-    if (page->layer(i) == page->active(view))
-      item->setBackgroundColor(Qt::yellow);
-    if (page->isLocked(i))
+    if (page->layer(i) == page->active(view)) {
+      item->setBackground(palette.highlight());
+      item->setForeground(palette.highlightedText());
+    }
+    if (page->isLocked(i)) {
       item->setBackground(QColor(255, 220, 220));
+      if (palette.text().color().value() > 128)
+	item->setForeground(QColor(48, 48, 48));
+    }
+    QColor bg = item->background() == Qt::NoBrush ? palette.base().color() : item->background().color();
     switch (page->snapping(i)) {
     case Page::SnapMode::Never:
-      item->setForeground(QColor(0, 0, 160));
+      if (bg.value() > 128)
+	item->setForeground(QColor(0, 0, 160));
+      else
+	item->setForeground(QColor(80, 160, 255));
       break;
     case Page::SnapMode::Always:
-      item->setForeground(QColor(0, 160, 0));
+      if (bg.value() > 128)
+	item->setForeground(QColor(0, 80, 0));
+      else
+	item->setForeground(QColor(0, 160, 0));
       break;
     default:
       break;
@@ -96,7 +110,7 @@ void LayerBox::set(const Page *page, int view)
 void LayerBox::mousePressEvent(QMouseEvent *ev)
 {
   LayerItem *item = (LayerItem *) itemAt(ev->pos());
-  if (item && ev->button()== Qt::LeftButton && ev->x() > 30) {
+  if (item && ev->button()== Qt::LeftButton && ev->position().x() > 30) {
     emit activated("active", item->ipeLayerName);
   }
   QListWidget::mousePressEvent(ev);
@@ -107,7 +121,7 @@ void LayerBox::mouseReleaseEvent(QMouseEvent *ev)
   LayerItem *item = (LayerItem *) itemAt(ev->pos());
   if (item && ev->button() == Qt::RightButton) {
     // make popup menu
-    emit showLayerBoxPopup(Vector(ev->globalPos().x(), ev->globalPos().y()),
+    emit showLayerBoxPopup(Vector(ev->globalPosition().x(), ev->globalPosition().y()),
 			   item->ipeLayerName);
   } else
     QListWidget::mouseReleaseEvent(ev);
@@ -115,15 +129,18 @@ void LayerBox::mouseReleaseEvent(QMouseEvent *ev)
 
 // --------------------------------------------------------------------
 
-#if QT_VERSION < 0x050000
-PathView::PathView(int uiScale, QWidget* parent, Qt::WFlags f) : QWidget(parent, f), iUiScale(uiScale)
-#else
-PathView::PathView(int uiScale, QWidget* parent, Qt::WindowFlags f) : QWidget(parent, f), iUiScale(uiScale)
-#endif
+PathView::PathView(int uiScale, QWidget* parent, Qt::WindowFlags f)
+  : QWidget(parent, f), iUiScale(uiScale)
 {
   iCascade = nullptr;
 }
 
+void PathView::setColor(const Color & color)
+{
+  iColor = color;
+  update();
+}
+
 void PathView::set(const AllAttributes &all, Cascade *sheet)
 {
   iCascade = sheet;
@@ -139,7 +156,9 @@ void PathView::paintEvent(QPaintEvent *ev)
 
   cairo_surface_t *sf = cairo_image_surface_create(CAIRO_FORMAT_RGB24, w, h);
   cairo_t *cc = cairo_create(sf);
-  cairo_set_source_rgb(cc, 1, 1, 0.8);
+  cairo_set_source_rgb(cc, iColor.iRed.toDouble(),
+		       iColor.iGreen.toDouble(),
+		       iColor.iBlue.toDouble());
   cairo_rectangle(cc, 0, 0, w, h);
   cairo_fill(cc);
 
@@ -157,7 +176,7 @@ void PathView::paintEvent(QPaintEvent *ev)
     Vector vf = iAll.iFArrowShape.isMidArrow() ? mid : v1;
     Vector vr = iAll.iRArrowShape.isMidArrow() ? mid : v0;
 
-    CairoPainter painter(iCascade, nullptr, cc, 3.0, false);
+    CairoPainter painter(iCascade, nullptr, cc, 3.0, false, false);
     painter.setPen(iAll.iPen);
     painter.setDashStyle(iAll.iDashStyle);
     painter.setStroke(iAll.iStroke);
@@ -206,14 +225,14 @@ QSize PathView::sizeHint() const
 void PathView::mousePressEvent(QMouseEvent *ev)
 {
   QSize s = size();
-  if (ev->button()== Qt::LeftButton && ev->x() < s.width() * 3 / 10) {
+  if (ev->button()== Qt::LeftButton && ev->position().x() < s.width() * 3 / 10) {
     emit activated(iAll.iRArrow ? "rarrow|false" : "rarrow|true");
   } else if (ev->button()== Qt::LeftButton &&
-	     ev->x() > s.width() * 4 / 10 &&
-	     ev->x() < s.width() * 72 / 100) {
+	     ev->position().x() > s.width() * 4 / 10 &&
+	     ev->position().x() < s.width() * 72 / 100) {
     emit activated(iAll.iFArrow ? "farrow|false" : "farrow|true");
     update();
-  } else if (ev->button()== Qt::LeftButton && ev->x() > s.width() * 78 / 100) {
+  } else if (ev->button()== Qt::LeftButton && ev->position().x() > s.width() * 78 / 100) {
     switch (iAll.iPathMode) {
     case EStrokedOnly:
       emit activated("pathmode|strokedfilled");
@@ -232,7 +251,7 @@ void PathView::mousePressEvent(QMouseEvent *ev)
 void PathView::mouseReleaseEvent(QMouseEvent *ev)
 {
   if (ev->button()== Qt::RightButton)
-    emit showPathStylePopup(Vector(ev->globalPos().x(), ev->globalPos().y()));
+    emit showPathStylePopup(Vector(ev->globalPosition().x(), ev->globalPosition().y()));
 }
 
 bool PathView::event(QEvent *ev)
@@ -288,10 +307,10 @@ PageSorter::PageSorter(Document *doc, int pno, int itemWidth, QWidget *parent)
       QString s;
       QString t = QString::fromUtf8(p->viewName(i).z());
       if (t != "") {
-	s.sprintf("%d: ", i+1);
+	s = QString("%1: ").arg(i+1);
 	s += t;
       } else {
-	s.sprintf("View %d", i+1);
+	s = QString("View %1").arg(i+1);
       }
       iMarks.push_back(p->markedView(i));
       QListWidgetItem *item = new QListWidgetItem(icon, s);
@@ -313,10 +332,10 @@ PageSorter::PageSorter(Document *doc, int pno, int itemWidth, QWidget *parent)
       QString s;
       QString t = QString::fromUtf8(p->title().z());
       if (t != "") {
-	s.sprintf("%d: ", i+1);
+	s = QString("%1: ").arg(i+1);
 	s += t;
       } else {
-	s.sprintf("Page %d", i+1);
+	s = QString("Page %1").arg(i+1);
       }
       iMarks.push_back(p->marked());
       QListWidgetItem *item = new QListWidgetItem(icon, s);
