@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2023 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -967,12 +967,15 @@ end
 
 function MODEL:action_insert_text_box()
   local layout = self.doc:sheets():find("layout")
+  local frame = ipe.Rect()
+  frame:add(V(-3, -3))
+  frame:add(layout.framesize + V(3, 3))
   local p = self:page()
   local r = ipe.Rect()
-  local m = ipe.Matrix()
   for i, obj, sel, layer in p:objects() do
-    if p:visible(self.vno, i) and obj:type() == "text" then
-      obj:addToBBox(r, m)
+    if p:visible(self.vno, i) and obj:type() == "text" and not p:isLocked(layer) then
+      local b = p:bbox(i)
+      if frame:contains(b) then r:add(b) end
     end
   end
   local y = layout.framesize.y
@@ -1094,6 +1097,14 @@ function GRAPHTOOL:new(model, moveInvisibleEdges)
   tool.model = model
   tool.orig = model.ui:pos()
   tool.box = p:bbox(self.prim)
+  if tool.box:width() < 6 then
+    tool.box:add(tool.box:topLeft() + V(-3, 0))
+    tool.box:add(tool.box:topRight() + V(3, 0))
+  end
+  if tool.box:height() < 6 then
+    tool.box:add(tool.box:topLeft() + V(0, 3))
+    tool.box:add(tool.box:bottomLeft() + V(0, -3))
+  end
   tool.edges = collect_edges(p, tool.box, view)
   model.ui:shapeTool(tool)
   tool.setColor(1.0, 0.0, 0.0)
@@ -1690,6 +1701,7 @@ function PASTETOOL:mouseButton(button, modifiers, press)
 	   end
   self.model:page():deselectAll()
   self.model:register(t)
+  self.model:autoRunLatex()
 end
 
 function PASTETOOL:mouseMove()
