@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -237,7 +237,7 @@ static bool applyPngPrediction(Buffer &data, int width, int height, int componen
   return true;
 }
 
-static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, double opacity)
+static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, double opacity, bool filterBest)
 {
   // ipeDebug("Image: %s", d->dictRepr().z());
   int width = d->getInteger("Width");
@@ -271,8 +271,10 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
   }
   Buffer alphaChannel;
   uint8_t *alpha = nullptr;
+  uint32_t colorKey = 0;
   const PdfDict *smask = r->getDict(d, "SMask");
   if (smask) {
+    // ipeDebug("Mask: %s", smask->dictRepr().z());
     const PdfObj *mcs = smask->get("ColorSpace");
     if (!mcs || !mcs->name() || mcs->name()->value() != "DeviceGray"
 	|| smask->getInteger("BitsPerComponent") != 8) {
@@ -282,6 +284,20 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
       if (alphaChannel.size() == width * height)
 	alpha = (uint8_t *) alphaChannel.data();
     }
+  } else {
+    std::vector<double> ckv;
+    if (d->getNumberArray("Mask", nullptr, ckv) &&
+	size(ckv) == 2 * components) {
+      if (components == 3) {
+	uint8_t r = uint8_t(ckv[0]);
+	uint8_t g = uint8_t(ckv[2]);
+	uint8_t b = uint8_t(ckv[4]);
+	colorKey = 0xff000000 | (r << 16) | (g << 8) | b;
+      } else {
+	uint8_t g = uint8_t(ckv[0]);
+	colorKey = 0xff000000 | (g << 16) | (g << 8) | g;
+      }
+    }
   }
   Buffer stream = d->inflate();
   Buffer pixels(4 * width * height);
@@ -300,11 +316,14 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
 	if (pngPrediction)
 	  ++p; // skip predictor byte
 	for (int x = 0; x < width; ++x) {
-	  uint8_t r = uint8_t(*p++);
-	  uint8_t g = uint8_t(*p++);
-	  uint8_t b = uint8_t(*p++);
 	  uint8_t a = alpha ? *alpha++ : 0xff;
+	  // premultiply with alpha
+	  uint8_t r = uint8_t(*p++) * a / 0xff;
+	  uint8_t g = uint8_t(*p++) * a / 0xff;
+	  uint8_t b = uint8_t(*p++) * a / 0xff;
 	  uint32_t pixel = (a << 24) | (r << 16) | (g << 8) | b;
+	  if (pixel == colorKey)
+	    pixel = 0x0;
 	  *q++ = pixel;
 	}
       }
@@ -313,24 +332,26 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
 	if (pngPrediction)
 	  ++p; // skip predictor byte
 	for (int x = 0; x < width; ++x) {
-	  uint8_t r = uint8_t(*p++);
 	  uint8_t a = alpha ? *alpha++ : 0xff;
+	  uint8_t r = uint8_t(*p++) * a / 0xff;
 	  uint32_t pixel = (a << 24) | (r << 16) | (r << 8) | r;
+	  if (pixel == colorKey)
+	    pixel = 0x0;
 	  *q++ = pixel;
 	}
       }
     }
   }
-  cairo_surface_t *image =
-    cairo_image_surface_create_for_data((uint8_t *) pixels.data(),
-					CAIRO_FORMAT_ARGB32,
-					width, height,
-					4 * width);
+  // we cannot use cairo_image_surface_create_for_data, because when rendering to PS or PDF,
+  // the surface is kept by cairo until showpage gets called, and pixels is local to this function.
+  cairo_surface_t *image = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
+  memcpy(cairo_image_surface_get_data(image), pixels.data(), 4 * width * height);
+  cairo_surface_mark_dirty(image);
   cairo_save(cr);
   Matrix tf = Matrix(1.0 / width, 0.0, 0.0, -1.0 / height, 0.0, 1.0);
   cairoTransform(cr, tf);
   cairo_set_source_surface(cr, image, 0, 0);
-  cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_FAST);
+  cairo_pattern_set_filter(cairo_get_source(cr), filterBest ? CAIRO_FILTER_BEST : CAIRO_FILTER_GOOD);
   cairo_paint_with_alpha(cr, opacity);
   cairo_restore(cr);
 }
@@ -348,8 +369,9 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, d
 /*! \a zoom one means 72 pixels per inch.  Set \a pretty to true
   to avoid drawing text without Latex. */
 CairoPainter::CairoPainter(const Cascade *sheet, Fonts *fonts,
-			   cairo_t *cc, double zoom, bool pretty)
-  : Painter(sheet), iFonts(fonts), iCairo(cc), iZoom(zoom), iPretty(pretty), iType3Font(false)
+			   cairo_t *cc, double zoom, bool pretty, bool filterBest)
+  : Painter(sheet), iFonts(fonts), iCairo(cc), iZoom(zoom),
+    iPretty(pretty), iFilterBest(filterBest), iType3Font(false)
 {
   iDimmed = false;
 }
@@ -465,14 +487,15 @@ void CairoPainter::doDrawPath(TPathMode mode)
       cairo_pattern_set_matrix(p, &m);
 
       cairo_set_source(iCairo, p);
+      cairo_pattern_destroy(p);  // pass ownership to iCairo
 
       if (mode == EStrokedAndFilled)
 	cairo_fill_preserve(iCairo);
       else
 	cairo_fill(iCairo);
 
-      cairo_pattern_destroy(p);
-
+      // release pattern
+      cairo_set_source_rgb(iCairo, 0.0, 0.0, 0.0);
     } else {
       // tiling
 
@@ -492,6 +515,7 @@ void CairoPainter::doDrawPath(TPathMode mode)
       cairo_fill(cc);
       cairo_destroy(cc);
       cairo_pattern_t *p = cairo_pattern_create_for_surface(s);
+      cairo_surface_destroy(s); // pass ownership to pattern
       cairo_pattern_set_extend(p, CAIRO_EXTEND_REPEAT);
 
       cairo_matrix_t m;
@@ -500,14 +524,15 @@ void CairoPainter::doDrawPath(TPathMode mode)
       cairo_pattern_set_matrix(p, &m);
 
       cairo_set_source(iCairo, p);
+      cairo_pattern_destroy(p); // pass ownership to iCairo
 
       if (mode == EStrokedAndFilled)
 	cairo_fill_preserve(iCairo);
       else
 	cairo_fill(iCairo);
 
-      cairo_pattern_destroy(p);
-      cairo_surface_destroy(s);
+      // release pattern so pattern and surface are destroyed
+      cairo_set_source_rgb(iCairo, 0.0, 0.0, 0.0);
     }
   }
 
@@ -549,7 +574,7 @@ void CairoPainter::doDrawPath(TPathMode mode)
       std::vector<double> dashes;
       double offset;
       dashStyle(dashes, offset);
-      cairo_set_dash(iCairo, &dashes[0], dashes.size(), offset);
+      cairo_set_dash(iCairo, dashes.data(), dashes.size(), offset);
     }
     cairo_stroke(iCairo);
   }
@@ -580,7 +605,8 @@ void CairoPainter::doDrawBitmap(Bitmap bitmap)
 				0.0, 1.0);
   cairoTransform(iCairo, tf);
   cairo_set_source_surface(iCairo, image, 0, 0);
-  cairo_pattern_set_filter(cairo_get_source(iCairo), CAIRO_FILTER_FAST);
+  cairo_pattern_set_filter(cairo_get_source(iCairo),
+			   iFilterBest ? CAIRO_FILTER_BEST : CAIRO_FILTER_GOOD);
   cairo_paint_with_alpha(iCairo, opacity().toDouble());
   cairo_restore(iCairo);
 }
@@ -700,6 +726,11 @@ void CairoPainter::execute(const PdfDict *xform, const PdfDict *resources, bool
       mx.a[i] = m[i];
     cairoTransform(iCairo, mx);
   }
+  std::vector<double> bbox;
+  if (xform->getNumberArray("BBox", nullptr, bbox) && bbox.size() == 4) {
+    cairo_rectangle(iCairo, bbox[0], bbox[1], bbox[2] - bbox[0], bbox[3] - bbox[1]);
+    cairo_clip(iCairo);
+  }
   Buffer buffer = xform->inflate();
   BufferSource source(buffer);
   PdfParser parser(source);
@@ -938,7 +969,7 @@ void CairoPainter::opd()
     dashes.emplace_back(obj->number()->value());
   }
   double offset = iArgs[1]->number()->value();
-  cairo_set_dash(iCairo, &dashes[0], dashes.size(), offset);
+  cairo_set_dash(iCairo, dashes.data(), dashes.size(), offset);
 }
 
 void CairoPainter::opi()
@@ -1035,7 +1066,7 @@ void CairoPainter::opDo()
     execute(xf, xf);
     cairo_restore(iCairo);
   } else if (subtype == "Image") {
-    drawImage(iCairo, xf, iFonts->resources(), iPdfState.back().iFillOpacity);
+    drawImage(iCairo, xf, iFonts->resources(), iPdfState.back().iFillOpacity, iFilterBest);
   } else
     ipeDebug("Do operator with unsupported XObject subtype %s", subtype.z());
 }
@@ -1192,7 +1223,7 @@ void CairoPainter::createPattern()
 	cairo_save(cc);
 	cairo_translate(cc, -dx * width, -dy * height);
 	cairo_scale(cc, xscale, yscale);
-	CairoPainter painter(iCascade, iFonts, cc, 1.0, false);
+	CairoPainter painter(iCascade, iFonts, cc, 1.0, false, false);
 	painter.iPdfState.push_back(ps);
 	painter.execute(pat, pat, false);
 	cairo_restore(cc);
@@ -1414,14 +1445,39 @@ void CairoPainter::drawGlyphs(std::vector<cairo_glyph_t> &glyphs)
   cairo_matrix_t matrix;
   cairoMatrix(matrix, m);
 
-  cairo_save(iCairo);
-  cairo_set_font_face(iCairo, ps.iFont->cairoFont());
-  cairo_set_font_matrix(iCairo, &matrix);
-  cairo_set_source_rgba(iCairo,
-			ps.iFillRgb[0], ps.iFillRgb[1], ps.iFillRgb[2],
-			ps.iFillOpacity);
-  cairo_show_glyphs(iCairo, &glyphs[0], glyphs.size());
-  cairo_restore(iCairo);
+  if (ps.iFont->type() == FontType::Type3) {
+    cairo_save(iCairo);
+    cairo_set_font_face(iCairo, Fonts::screenFont());
+    cairo_set_source_rgba(iCairo,
+			  ps.iFillRgb[0], ps.iFillRgb[1], ps.iFillRgb[2],
+			  0.5);
+    cairo_save(iCairo);
+    cairo_set_font_matrix(iCairo, &matrix);
+    cairo_show_glyphs(iCairo, glyphs.data(), glyphs.size());
+    cairo_restore(iCairo);
+    double s = ps.iFontSize;
+    cairo_set_font_size(iCairo, 0.23 * s);
+    for (int i = 0; i < size(glyphs); ++i) {
+      auto pt = iTextMatrix * Vector(glyphs[i].x, glyphs[i].y);
+      cairo_save(iCairo);
+      cairo_translate(iCairo, pt.x, pt.y);
+      cairo_rotate(iCairo, 0.4 * IPE_PI);
+      cairo_scale(iCairo, 1.0, -1.0);
+      cairo_move_to(iCairo, -0.05 * s, 0.3 * s);
+      cairo_show_text(iCairo, "Type3");
+      cairo_restore(iCairo);
+    }
+    cairo_restore(iCairo);
+  } else {
+    cairo_save(iCairo);
+    cairo_set_font_face(iCairo, ps.iFont->cairoFont());
+    cairo_set_font_matrix(iCairo, &matrix);
+    cairo_set_source_rgba(iCairo,
+			  ps.iFillRgb[0], ps.iFillRgb[1], ps.iFillRgb[2],
+			  ps.iFillOpacity);
+    cairo_show_glyphs(iCairo, glyphs.data(), glyphs.size());
+    cairo_restore(iCairo);
+  }
 }
 
 // --------------------------------------------------------------------
