@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -315,6 +315,23 @@ void String::append(char ch) noexcept
   iImp->iData[iImp->iSize++] = ch;
 }
 
+//! Append a single unicode character \a ch in UTF-8 encoding.
+void String::appendUtf8(uint16_t ch) noexcept
+{
+  if (ch < 0x80) {
+    append(char(ch));
+  } else if (ch < 0x800) {
+    detach(2);
+    iImp->iData[iImp->iSize++] = 0xc0 | (ch >> 6);
+    iImp->iData[iImp->iSize++] = 0x80 | (ch & 0x3f);
+  } else {
+    detach(3);
+    iImp->iData[iImp->iSize++] = 0xe0 | (ch >> 12);
+    iImp->iData[iImp->iSize++] = 0x80 | ((ch >> 6) & 0x3f);
+    iImp->iData[iImp->iSize++] = 0x80 | (ch & 0x3f);
+  }
+}
+
 //! Create substring at the right.
 /*! Returns the entire string if \a i is larger than its length. */
 String String::right(int i) const noexcept
@@ -784,6 +801,29 @@ DataSource::~DataSource()
   // nothing
 }
 
+//! Return length of input stream in characters.
+/*! Returns -1 if the stream is not seekable.
+  Calling this function will invalidate the current position.
+*/
+int DataSource::length() const
+{
+  return -1;
+}
+
+//! Set position in stream.
+/*! Does nothing if the stream is not seekable. */
+void DataSource::setPosition(int pos)
+{
+  // do nothing
+}
+
+//! Return position in stream.
+/*! Returns -1 if the stream is not seekable. */
+int DataSource::position() const
+{
+  return -1;
+}
+
 // --------------------------------------------------------------------
 
 /*! \class ipe::FileSource
@@ -802,6 +842,26 @@ int FileSource::getChar()
   return std::fgetc(iFile);
 }
 
+int FileSource::length() const
+{
+  if (std::fseek(iFile, 0L, SEEK_END) == 0)
+    return std::ftell(iFile);
+  else
+    return -1;
+}
+
+void FileSource::setPosition(int pos)
+{
+  std::fseek(iFile, pos, SEEK_SET);
+}
+
+int FileSource::position() const
+{
+  return std::ftell(iFile);
+}
+
+// --------------------------------------------------------------------
+
 /*! \class ipe::BufferSource
   \ingroup base
   \brief Data source for parsing from a buffer.
@@ -813,11 +873,6 @@ BufferSource::BufferSource(const Buffer &buffer)
   iPos = 0;
 }
 
-void BufferSource::setPosition(int pos)
-{
-  iPos = pos;
-}
-
 int BufferSource::getChar()
 {
   if (iPos >= iBuffer.size())
@@ -825,4 +880,19 @@ int BufferSource::getChar()
   return uint8_t(iBuffer[iPos++]);
 }
 
+int BufferSource::length() const
+{
+  return iBuffer.size();
+}
+
+void BufferSource::setPosition(int pos)
+{
+  iPos = pos;
+}
+
+int BufferSource::position() const
+{
+  return iPos;
+}
+
 // --------------------------------------------------------------------
