@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2023 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -183,6 +183,16 @@ void Latex::addPageNumber(int pno, int vno, int npages, int nviews)
 int Latex::createLatexSource(Stream &stream, String preamble)
 {
   int count = 0;
+  if (preamble.hasPrefix("%&")) {
+    int i = preamble.find('\n');
+    if (i < 0) {
+      stream << preamble << "\n";
+      preamble.erase();
+    } else {
+      stream << preamble.left(i+1);
+      preamble = preamble.substr(i+1);
+    }
+  }
   stream << "\\nonstopmode\n";
   if (!iXetex) {
     // for parser testing: \\pdfobjcompresslevel2\\pdfminorversion5
