@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2023 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -226,7 +226,9 @@ void PdfPainter::doDrawText(const Text *text)
 
 void PdfPainter::doDrawSymbol(Attribute symbol)
 {
-  const Symbol *sym = cascade()->findSymbol(symbol);
+  const Symbol *sym = iAttributeMap ?
+    cascade()->findSymbol(iAttributeMap->map(ESymbol, symbol)) :
+    cascade()->findSymbol(symbol);
   if (!sym)
     return;
   if (sym->iXForm)
