@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2023 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -180,7 +180,7 @@ void Presenter::collectPageLabels(const PdfDict *d)
       const PdfObj *p = label->dict()->get("P", iPdf.get());
       String newLabel;
       if (p && p->string())
-	newLabel = p->string()->value();
+	newLabel = p->string()->decode();
       bool moreThanOne = (newNum - prevNum) > 1;
       while (size(iPageLabels) < newNum)
 	iPageLabels.push_back(std::make_pair(prevLabel, moreThanOne ?
