@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2023 Otfried Cheong
+    Copyright (c) 1993-2024 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -611,25 +611,13 @@ static int menu_constructor(lua_State *L)
 
 // --------------------------------------------------------------------
 
-class EditorThread : public QThread
-{
-public:
-  EditorThread(lua_State *L, const QString &cmd);
-
-protected:
-  virtual void run();
-private:
-  lua_State *L;
-  QString iCommand;
-};
-
-EditorThread::EditorThread(lua_State *L, const QString &cmd)
-  : QThread(), L{L}, iCommand{cmd}
+Waiter::Waiter(lua_State *L, const QString &cmd)
+  : L{L}, iCommand{cmd}
 {
   // nothing
 }
 
-void EditorThread::run()
+void Waiter::process()
 {
   if (iCommand.isEmpty()) {
     lua_call(L, 0, 0);
@@ -640,19 +628,13 @@ void EditorThread::run()
     int result = std::system(iCommand.toUtf8());
     (void) result;
   }
+  emit completed();
 }
 
-class EditorDialog : public QDialog
-{
-public:
-  EditorDialog(QString label, QWidget *parent = nullptr);
-protected:
-  void keyPressEvent(QKeyEvent *e);
-  void closeEvent(QCloseEvent *ev);
-};
+// --------------------------------------------------------------------
 
-EditorDialog::EditorDialog(QString label, QWidget *parent)
-  : QDialog(parent)
+WaitDialog::WaitDialog(QString label, QWidget *parent)
+  : QDialog{parent}, running{true}
 {
   QGridLayout *lo = new QGridLayout;
   setLayout(lo);
@@ -661,19 +643,40 @@ EditorDialog::EditorDialog(QString label, QWidget *parent)
   lo->addWidget(l, 0, 0);
 }
 
-void EditorDialog::keyPressEvent(QKeyEvent *e)
+void WaitDialog::keyPressEvent(QKeyEvent *e)
 {
   // do not let Escape close the dialog
   if (e->key() != Qt::Key_Escape)
     QDialog::keyPressEvent(e);
 }
 
-void EditorDialog::closeEvent(QCloseEvent *ev)
+void WaitDialog::closeEvent(QCloseEvent *ev)
 {
   // do not let user close the dialog
   ev->ignore();
 }
 
+// start modal dialog if the thread has not yet signaled and keep mutex locked
+void WaitDialog::startDialog()
+{
+  mutex.lock();
+  if (running)
+    this->exec();
+  mutex.unlock();
+};
+
+void WaitDialog::completed()
+{
+  if (mutex.tryLock()) {
+    // the mutex was free, so dialog is not yet showing
+    running = false;
+    mutex.unlock();
+  } else
+    done(0);
+}
+
+// --------------------------------------------------------------------
+
 static int ipeui_wait(lua_State *L)
 {
   PDialog *parent = nullptr;
@@ -687,27 +690,34 @@ static int ipeui_wait(lua_State *L)
   if (lua_isstring(L, 3))
     label = checkqstring(L, 3);
 
-  EditorThread *thread = new EditorThread(L, cmd);
-  EditorDialog *dialog = new EditorDialog(label);
+  QThread *thread = new QThread();
+  Waiter *waiter = new Waiter(L, cmd);
+  waiter->moveToThread(thread);
+  WaitDialog *dialog = new WaitDialog(label);
+
+  // waiter is in a different thread, but connect does this right
+  QObject::connect(thread, &QThread::started, waiter, &Waiter::process);
+  QObject::connect(waiter, &Waiter::completed, thread, &QThread::quit);
+  QObject::connect(waiter, &Waiter::completed, waiter, &QObject::deleteLater);
+  QObject::connect(thread, &QThread::finished, thread, &QObject::deleteLater);
+  QObject::connect(waiter, &Waiter::completed, dialog, &WaitDialog::completed);
 
   if (cmd.isEmpty()) {
     // call Lua function
     lua_pushvalue(L, 2);
     thread->start();
-    if (!thread->wait(300)) {
-      dialog->connect(thread, &QThread::finished, dialog, &QDialog::accept);
-      dialog->exec();
+    for (int i = 0; i < 3 && dialog->isRunning(); ++i) {
+      QThread::msleep(100);
+      QCoreApplication::processEvents();
     }
+    dialog->startDialog();
   } else {
-    dialog->connect(thread, &QThread::finished, dialog, &QDialog::accept);
     thread->start();
-    dialog->exec();
+    dialog->startDialog();
     if (parent)
       parent->activateWindow();
   }
-
   delete dialog;
-  delete thread;
   return 0;
 }
 
